@@ -1,7 +1,7 @@
 /*
  * thumbrecoil.cpp - Windows thumbnail provider for RECOIL
  *
- * Copyright (C) 2011-2015  Piotr Fusik
+ * Copyright (C) 2011-2018  Piotr Fusik
  *
  * This file is part of RECOIL (Retro Computer Image Library),
  * see http://recoil.sourceforge.net
@@ -28,45 +28,8 @@
 
 #include <windows.h>
 #include <objidl.h>
-
-#undef INTERFACE
-
-#ifdef __MINGW64__
-
 #include <shobjidl.h>
 
-#else
-// missing just in 32-bit MinGW
-
-#define IEIFLAG_CACHE 2
-
-const IID IID_IExtractImage =
-	{ 0xbb2e617c, 0x0920, 0x11d1, { 0x9a, 0x0b, 0x00, 0xc0, 0x4f, 0xc2, 0xd6, 0xc1 } };
-#define INTERFACE IExtractImage
-DECLARE_INTERFACE_(IExtractImage, IUnknown)
-{
-	STDMETHOD(QueryInterface)(THIS_ REFIID, PVOID *) PURE;
-	STDMETHOD_(ULONG, AddRef)(THIS) PURE;
-	STDMETHOD_(ULONG, Release)(THIS) PURE;
-	STDMETHOD(GetLocation)(THIS_ LPWSTR, DWORD, DWORD *, const SIZE *, DWORD, DWORD *) PURE;
-	STDMETHOD(Extract)(THIS_ HBITMAP *) PURE;
-};
-#undef INTERFACE
-
-static const IID IID_IInitializeWithStream =
-	{ 0xb824b49d, 0x22ac, 0x4161, { 0xac, 0x8a, 0x99, 0x16, 0xe8, 0xfa, 0x3f, 0x7f } };
-#define INTERFACE IInitializeWithStream
-DECLARE_INTERFACE_(IInitializeWithStream, IUnknown)
-{
-	STDMETHOD(QueryInterface)(THIS_ REFIID, PVOID *) PURE;
-	STDMETHOD_(ULONG, AddRef)(THIS) PURE;
-	STDMETHOD_(ULONG, Release)(THIS) PURE;
-	STDMETHOD(Initialize)(THIS_ IStream *, DWORD) PURE;
-};
-#undef INTERFACE
-
-#endif
-
 enum WTS_ALPHATYPE
 {
 	WTSAT_UNKNOWN = 0,
@@ -76,6 +39,7 @@ enum WTS_ALPHATYPE
 
 static const IID IID_IThumbnailProvider =
 	{ 0xe357fccd, 0xa995, 0x4576, { 0xb0, 0x1f, 0x23, 0x46, 0x30, 0x15, 0x4e, 0x96 } };
+#undef INTERFACE
 #define INTERFACE IThumbnailProvider
 DECLARE_INTERFACE_(IThumbnailProvider, IUnknown)
 {
@@ -90,7 +54,6 @@ DECLARE_INTERFACE_(IThumbnailProvider, IUnknown)
 #include "formats.h"
 
 static const char extensions[][6] = { THUMBRECOIL_EXTS };
-#define N_EXTS (int) (sizeof(extensions) / sizeof(extensions[0]))
 
 static HINSTANCE g_hDll;
 static LONG g_cRef = 0;
@@ -145,7 +108,7 @@ class CRECOILThumbProvider : IPersistFile, IExtractImage
 		bmi.bmiHeader.biBitCount = 32;
 		bmi.bmiHeader.biCompression = BI_RGB;
 		int *pBits;
-		HBITMAP hbmp = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, (void **) &pBits, NULL, 0);
+		HBITMAP hbmp = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, reinterpret_cast<void **>(&pBits), NULL, 0);
 		if (hbmp == NULL)
 			return E_OUTOFMEMORY;
 		int pixels_count = width * height;
@@ -175,23 +138,23 @@ public:
 	STDMETHODIMP QueryInterface(REFIID riid, void **ppv)
 	{
 		if (riid == IID_IUnknown || riid == IID_IPersistFile) {
-			*ppv = (IPersistFile *) this;
+			*ppv = static_cast<IPersistFile *>(this);
 			AddRef();
 			return S_OK;
 		}
 		if (riid == IID_IExtractImage) {
-			*ppv = (IExtractImage *) this;
+			*ppv = static_cast<IExtractImage *>(this);
 			AddRef();
 			return S_OK;
 		}
 #if THUMBRECOIL_VISTA
 		if (riid == IID_IInitializeWithStream) {
-			*ppv = (IInitializeWithStream *) this;
+			*ppv = static_cast<IInitializeWithStream *>(this);
 			AddRef();
 			return S_OK;
 		}
 		if (riid == IID_IThumbnailProvider) {
-			*ppv = (IThumbnailProvider *) this;
+			*ppv = static_cast<IThumbnailProvider *>(this);
 			AddRef();
 			return S_OK;
 		}
@@ -236,7 +199,7 @@ public:
 			return HRESULT_FROM_WIN32(GetLastError());
 		free(m_filename);
 		m_filename = NULL;
-		if (!ReadFile(fh, m_content, sizeof(m_content), (LPDWORD) &m_contentLen, NULL)) {
+		if (!ReadFile(fh, m_content, sizeof(m_content), reinterpret_cast<LPDWORD>(&m_contentLen), NULL)) {
 			HRESULT hr = HRESULT_FROM_WIN32(GetLastError());
 			CloseHandle(fh);
 			return hr;
@@ -314,7 +277,7 @@ public:
 			return hr;
 
 		// get contents
-		hr = m_pstream->Read(m_content, sizeof(m_content), (ULONG *) &m_contentLen);
+		hr = m_pstream->Read(m_content, sizeof(m_content), reinterpret_cast<ULONG *>(&m_contentLen));
 		if (FAILED(hr)) {
 			CoTaskMemFree(statstg.pwcsName);
 			return hr;
@@ -337,7 +300,7 @@ public:
 	STDMETHODIMP QueryInterface(REFIID riid, void **ppv)
 	{
 		if (riid == IID_IUnknown || riid == IID_IClassFactory) {
-			*ppv = (IClassFactory *) this;
+			*ppv = static_cast<IClassFactory *>(this);
 			DllAddRef();
 			return S_OK;
 		}
@@ -377,7 +340,7 @@ public:
 		else
 			DllRelease();
 		return S_OK;
-	};
+	}
 };
 
 STDAPI_(BOOL) DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
@@ -387,62 +350,65 @@ STDAPI_(BOOL) DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
 	return TRUE;
 }
 
+static BOOL MyRegCreateKey(HKEY hk1, LPCSTR subkey, PHKEY hk2)
+{
+	return RegCreateKeyEx(hk1, subkey, 0, NULL, 0, KEY_WRITE, NULL, hk2, NULL) == ERROR_SUCCESS;
+}
+
+static BOOL MyRegSetValueString(HKEY hk1, LPCSTR name, LPCSTR data)
+{
+	return RegSetValueEx(hk1, name, 0, REG_SZ, reinterpret_cast<const BYTE *>(data), strlen(data) + 1) == ERROR_SUCCESS;
+}
+
 static BOOL RegisterCLSID(HKEY hk1, LPCSTR subkey)
 {
 	HKEY hk2;
-	if (RegCreateKeyEx(hk1, subkey, 0, NULL, 0, KEY_WRITE, NULL, &hk2, NULL) != ERROR_SUCCESS)
+	if (!MyRegCreateKey(hk1, subkey, &hk2))
 		return FALSE;
-	static const char CLSID_RECOILThumbProvider_str2[] = CLSID_RECOILThumbProvider_str;
-	BOOL ok = RegSetValueEx(hk2, NULL, 0, REG_SZ, (CONST BYTE *) CLSID_RECOILThumbProvider_str2, sizeof(CLSID_RECOILThumbProvider_str2)) == ERROR_SUCCESS;
+	BOOL ok = MyRegSetValueString(hk2, NULL, CLSID_RECOILThumbProvider_str);
 	RegCloseKey(hk2);
 	return ok;
 }
 
 STDAPI __declspec(dllexport) DllRegisterServer(void)
 {
+	char szModulePath[MAX_PATH];
+	if (GetModuleFileName(g_hDll, szModulePath, MAX_PATH) == 0)
+		return E_FAIL;
 	HKEY hk1;
-	if (RegCreateKeyEx(HKEY_CLASSES_ROOT, "CLSID\\" CLSID_RECOILThumbProvider_str, 0, NULL, 0, KEY_WRITE, NULL, &hk1, NULL) != ERROR_SUCCESS)
+	if (!MyRegCreateKey(HKEY_CLASSES_ROOT, "CLSID\\" CLSID_RECOILThumbProvider_str, &hk1))
 		return E_FAIL;
 	HKEY hk2;
-	if (RegCreateKeyEx(hk1, "InProcServer32", 0, NULL, 0, KEY_WRITE, NULL, &hk2, NULL) != ERROR_SUCCESS) {
-		RegCloseKey(hk1);
-		return E_FAIL;
-	}
-	char szModulePath[MAX_PATH];
-	DWORD nModulePathLen = GetModuleFileName(g_hDll, szModulePath, MAX_PATH);
-	static const char szThreadingModel[] = "Both";
-	if (RegSetValueEx(hk2, NULL, 0, REG_SZ, (CONST BYTE *) szModulePath, nModulePathLen) != ERROR_SUCCESS
-	 || RegSetValueEx(hk2, "ThreadingModel", 0, REG_SZ, (CONST BYTE *) szThreadingModel, sizeof(szThreadingModel)) != ERROR_SUCCESS) {
-		RegCloseKey(hk2);
+	if (!MyRegCreateKey(hk1, "InProcServer32", &hk2)) {
 		RegCloseKey(hk1);
 		return E_FAIL;
 	}
+	BOOL ok = MyRegSetValueString(hk2, NULL, szModulePath)
+		&& MyRegSetValueString(hk2, "ThreadingModel", "Both");
 	RegCloseKey(hk2);
 	RegCloseKey(hk1);
+	if (!ok)
+		return E_FAIL;
 
-	for (int i = 0; i < N_EXTS; i++) {
-		if (RegCreateKeyEx(HKEY_CLASSES_ROOT, extensions[i], 0, NULL, 0, KEY_WRITE, NULL, &hk1, NULL) != ERROR_SUCCESS)
+	for (LPCSTR ext : extensions) {
+		if (!MyRegCreateKey(HKEY_CLASSES_ROOT, ext, &hk1))
 			return E_FAIL;
-		if (!RegisterCLSID(hk1, "ShellEx\\{bb2e617c-0920-11d1-9a0b-00c04fc2d6c1}") // IPersistFile+IExtractImage
+		ok = RegisterCLSID(hk1, "ShellEx\\{bb2e617c-0920-11d1-9a0b-00c04fc2d6c1}") // IPersistFile+IExtractImage
 #if THUMBRECOIL_VISTA
-		 || !RegisterCLSID(hk1, "ShellEx\\{e357fccd-a995-4576-b01f-234630154e96}") // IInitializeWithStream+IThumbnailProvider
+			&& RegisterCLSID(hk1, "ShellEx\\{e357fccd-a995-4576-b01f-234630154e96}") // IInitializeWithStream+IThumbnailProvider
 #endif
-		)
-		{
-			RegCloseKey(hk1);
-			return E_FAIL;
-		}
+			&& MyRegSetValueString(hk1, "PerceivedType", "image");
 		RegCloseKey(hk1);
+		if (!ok)
+			return E_FAIL;
 	}
 
 	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved", 0, KEY_SET_VALUE, &hk1) != ERROR_SUCCESS)
 		return E_FAIL;
-	static const char szDescription[] = "RECOIL Thumbnail Handler";
-	if (RegSetValueEx(hk1, CLSID_RECOILThumbProvider_str, 0, REG_SZ, (CONST BYTE *) szDescription, sizeof(szDescription)) != ERROR_SUCCESS) {
-		RegCloseKey(hk1);
-		return E_FAIL;
-	}
+	ok = MyRegSetValueString(hk1, CLSID_RECOILThumbProvider_str, "RECOIL Thumbnail Handler");
 	RegCloseKey(hk1);
+	if (!ok)
+		return E_FAIL;
 	return S_OK;
 }
 
@@ -453,15 +419,15 @@ STDAPI __declspec(dllexport) DllUnregisterServer(void)
 		RegDeleteValue(hk1, CLSID_RECOILThumbProvider_str);
 		RegCloseKey(hk1);
 	}
-	for (int i = 0; i < N_EXTS; i++) {
-		if (RegOpenKeyEx(HKEY_CLASSES_ROOT, extensions[i], 0, DELETE, &hk1) == ERROR_SUCCESS) {
+	for (LPCSTR ext : extensions) {
+		if (RegOpenKeyEx(HKEY_CLASSES_ROOT, ext, 0, DELETE, &hk1) == ERROR_SUCCESS) {
 			RegDeleteKey(hk1, "ShellEx\\{bb2e617c-0920-11d1-9a0b-00c04fc2d6c1}"); // IPersistFile+IExtractImage
 #if THUMBRECOIL_VISTA
 			RegDeleteKey(hk1, "ShellEx\\{e357fccd-a995-4576-b01f-234630154e96}"); // IInitializeWithStream+IThumbnailProvider
 #endif
 			RegCloseKey(hk1);
 		}
-		RegDeleteKey(HKEY_CLASSES_ROOT, extensions[i]);
+		RegDeleteKey(HKEY_CLASSES_ROOT, ext);
 	}
 	RegDeleteKey(HKEY_CLASSES_ROOT, "CLSID\\" CLSID_RECOILThumbProvider_str "\\InProcServer32");
 	RegDeleteKey(HKEY_CLASSES_ROOT, "CLSID\\" CLSID_RECOILThumbProvider_str);
